#######################################################################################
    # This script consist of the estimation of the model of house prices ###
    # For the sample 1990-04-01 to 2004-01-01, change the sample restriction, and use EXPEC (instead of EXPEC2)
    # The following code is based on teaching material at BI during the master's degree. 
    # For this estimation, use the file: "Dataset_Reestimation of Jacobsen & Naug (2004)"
#######################################################################################

# Packages
library(rmarkdown)  # to generate pdf file
library(tinytex)    # to generate pdf file
library(readxl)     # to read Excel files
library(urca)       # to perform ADF test
library(lmtest)     # to generate tests
library(ggplot2)    # plot time series 
library(sandwich)   # to estimate regressions using Newey-West standard errors 

# Working directory
setwd("~/Desktop/Koding")

# Import data from Excel file
data <- read_excel("/Users/piaegeberg/Desktop/Koding/Dataset_Reestimation of Jacobsen & Naug (2004).xlsx")
data <- as.data.frame(data)

# Fixing the colmns
colnames(data) <- c("month", "housingstock", "houseprice", "hprice_s.adj", "income", "disp.inc", "KPI", 
                    "KPI-JAE", "unem","interest.pre.tax" ,"taxrate", "TNSG", "S1", "S2", "S3", "INTEREST", "EXPEC","EXPEC2", "dispinc", "changehp", "changedis", "changeINT")

# Data as month
data$month <- as.Date(data$month)
data$TNSG <- as.numeric(data$TNSG)

# Measured as a rate 
data$TNSG <- data$TNSG/100 

#######################################################################################

####### Defining the variables ####### 

# logs
data$l.houseprice <- log(data$houseprice)
data$l.unemployment <- log(data$unem)
data$l.income <- log(data$income)
data$l.housingstock <- log(data$housingstock)
data$l.INTEREST <- log(data$INTEREST)


# differenced versions of the variables 
data$d.INTEREST <- c( NA, diff(data$INTEREST))
data$d.houseprice <- c(NA, diff(data$l.houseprice))
data$dd.houseprice <- c(NA, diff(data$houseprice))
data$d.income <- c( NA, diff(data$l.income))  
data$d.unemployment <- c( NA, diff(data$l.unem))  
data$d.l.INTEREST <-  c(NA, diff(data$l.INTEREST))
data$d.housingstock <- c(NA, diff(data$housingstock))

# lagged variables
data$d.INTEREST_1 <- c( 0, head(data$d.INTEREST, -1))
data$l.houseprice_1 <- c( 0, head(data$l.houseprice, -1))
data$unem_1 <- c( 0, head(data$l.unem, -1))

data$d.l.hprice <-  c( NA, diff(data$l.houseprice_1)) 

# lagged levels
data$houseprice_1 <-  c( 0, head(data$houseprice, -1))
data$INTEREST_1 <- c( 0, head(data$INTEREST, -1))

# creating income-housingstock
data$income.min.housingstock <- data$l.income - data$l.housingstock
data$l.housingstock_1 <- c( 0, head(data$l.housingstock, -1))
data$l.income_1 <-  c( 0, head(data$l.income, -1))
data$income.min.housingmass_1 <- data$l.income_1 - data$l.housingstock_1

################################################################################

# Restricting the sample period (either to 2022-10-01 or 2004-01-01 )
data <- data[ (data$month >= "1990-04-01")  &  (data$month <= "2022-10-01"), ]


################################################################################
########################## Plotting the time series ############################ 
################################################################################

# Plotting the time series
par(mfrow   = c(2,3), 
    pty     = "s",
    las     = 0,
    mgp     = c(4, 1, 0), 
    mar     = c(4, 3, 2, 2), 
    family  = "serif",
    cex.axis = 1.3)
plot(data$month, data$houseprice, 
     type = "l", 
     main = "Price index for resale homes",
     col  = "blue",
     xlab = "", 
     ylab = "")
grid()
plot(data$month, data$unem, 
     type = "l", 
     main = "Unemployment rate",
     col  = "blue",
     xlab = "", 
     ylab = "")
grid()
plot(data$month, data$income, 
     type = "l",
     main = "Household (wage) income",
     col  = "blue",
     xlab = "", 
     ylab = "")
grid()
plot(data$month, data$housingstock, 
     type = "l", 
     main = "Housing stock",
     col  = "blue",
     xlab = "", 
     ylab = "")
grid()
plot(data$month, data$INTEREST, 
     type = "l",
     main = "Banks average lending rate after tax",
     col  = "blue",
     xlab = "", 
     ylab = "")
grid()
plot(data$month, data$TNSG, 
     type = "l", 
     main = "TNS Gallup's expectation indicator",
     col  = "blue",
     xlab = "", 
     ylab = "")
grid()


################################################################################
########################## CHECKING FOR STATIONARITY ########################### 
################################################################################

############################# Checking log income ##############################

#ADF test with automatic lag selection with up to 12 lags according to BIC
adf_l.inc <- ur.df( data$l.income, type="none", lags = 12, selectlags = c("BIC") )
summary(adf_l.inc)

# Null hypothesis and alternative hypothesis
# H0: non-stationary with unit root 
# H1: stationary AR model with drift 

# Calculating the test statistic
l.inc.hyp.test_stat <- summary(adf_l.inc)@teststat[1]
print(l.inc.hyp.test_stat)

# Calculating critical value at 5% signficance level
l.inc.hyp.crit_value <- summary(adf_l.inc)@cval[1,2]
print(l.inc.hyp.crit_value)

# The rejection rule
if ( l.inc.hyp.test_stat < l.inc.hyp.crit_value ) {
  l.inc.hyp.reject <- "reject H0"
} else {
  l.inc.hyp.reject <- "do not reject H0"
}
print(l.inc.hyp.reject)

## rejection rule: test stat must be more negative 
## RESULT: cannot reject H0 of non-stationarity for income


########################## Checking log unemployment ###########################

#ADF test with automatic lag selection with up to 12 lags according to BIC
adf_l.unem. <- ur.df( data$l.unemployment, type="none", lags = 12, selectlags = c("BIC") )
summary(adf_l.unem.)

# Null hypothesis and alternative hypothesis
# H0: non-stationary with unit root 
# H1: stationary AR model with drift 

# Calculating the test statistic
l.unem.hyp.test_stat <- summary(adf_l.unem.)@teststat[1]
print(l.unem.hyp.test_stat)

# Calculating the critical value at 5% signficance level
l.unem.hyp.crit_value <- summary(adf_l.unem.)@cval[1,2]
print(l.unem.hyp.crit_value)

# The rejection rule
if ( l.unem.hyp.test_stat < l.unem.hyp.crit_value ) {
  l.unem.hyp.reject <- "reject H0"
} else {
  l.unem.hyp.reject <- "do not reject H0"
}
print(l.unem.hyp.reject)

## rejection rule: test stat must be more negative 
## RESULT: cannot reject H0 of non-stationarity for unempoloyment 


########################## Checking log house price ###########################

#ADF test with automatic lag selection with up to 12 lags according to BIC
adf_l.hp <- ur.df( data$l.houseprice, type="none", lags = 12, selectlags = c("BIC") )
summary(adf_l.hp)

# Null hypothesis and alternative hypothesis
# H0: non-stationary with unit root 
# H1: stationary AR model with drift 

# Calculating the test statistic
l.hp.hyp.test_stat <- summary(adf_l.hp)@teststat[1]
print(l.hp.hyp.test_stat)

# Calculating the critical value at 5% signficance level
l.hp.hyp.crit_value <- summary(adf_l.hp)@cval[1,2]
print(l.hp.hyp.crit_value)

# The rejection rule
if ( l.hp.hyp.test_stat < l.hp.hyp.crit_value ) {
  l.hp.hyp.reject <- "reject H0"
} else {
  l.hp.hyp.reject <- "do not reject H0"
}
print(l.hp.hyp.reject)

## rejection rule: test stat must be more negative 
## RESULT: cannot reject H0 of non-stationarity for house price 


########################## Checking log housing stock ###########################

#ADF test with automatic lag selection with up to 12 lags according to BIC
adf_l.hs <- ur.df( data$l.housingstock, type="none", lags = 12, selectlags = c("BIC") )
summary(adf_l.hs)

# Null hypothesis and alternative hypothesis
# H0: non-stationary with unit root 
# H1: stationary AR model with drift 

# Calculating the test statistic
l.hs.hyp.test_stat <- summary(adf_l.hs)@teststat[1]
print(l.hs.hyp.test_stat)

# Calculating the critical value at 5% signficance level
l.hs.hyp.crit_value <- summary(adf_l.hs)@cval[1,2]
print(l.hs.hyp.crit_value)

# The rejection rule
if ( l.hs.hyp.test_stat < l.hs.hyp.crit_value ) {
  l.hs.hyp.reject <- "reject H0"
} else {
  l.hs.hyp.reject <- "do not reject H0"
}
print(l.hs.hyp.reject)

## rejection rule: test stat must be more negative 
## RESULT: cannot reject H0 of non-stationarity for housing stock 


########################## Checking INTEREST after tax ###########################

#ADF test with automatic lag selection with up to 12 lags according to BIC
adf_INT <- ur.df( data$INTEREST, type="none", lags = 12, selectlags = c("BIC") )
summary(adf_INT)

# Null hypothesis and alternative hypothesis
# H0: non-stationary with unit root 
# H1: stationary AR model with drift 

# Calculating the test statistic
INT.hyp.test_stat <- summary(adf_INT)@teststat[1]
print(INT.hyp.test_stat)

# Calculate the critical value at 5% signficance level
INT.hyp.crit_value <- summary(adf_INT)@cval[1,2]
print(INT.hyp.crit_value)

# The rejection rule
if ( INT.hyp.test_stat < INT.hyp.crit_value ) {
  INT.hyp.reject <- "reject H0"
} else {
  INT.hyp.reject <- "do not reject H0"
}
print(INT.hyp.reject)

## rejection rule: test stat must be more negative 
## RESULT: cannot reject H0 of non-stationarity for INTEREST after tax 


########################## Checking EXPEC2 ###########################

#ADF test with automatic lag selection with up to 12 lags according to BIC
adf_EXP <- ur.df( data$EXPEC2, type="none", lags = 12, selectlags = c("BIC") ) #EXPEC if =>2004
summary(adf_EXP)


# Null hypothesis and alternative hypothesis
# H0: non-stationary with unit root 
# H1: stationary AR model with drift 

# Calculating the test statistic
EXP.hyp.test_stat <- summary(adf_EXP)@teststat[1]
print(EXP.hyp.test_stat)

# Calculating the critical value at 5% signficance level
EXP.hyp.crit_value <- summary(adf_EXP)@cval[1,2]
print(EXP.hyp.crit_value)

# Calculating the rejection rule
if ( EXP.hyp.test_stat < EXP.hyp.crit_value ) {
  EXP.hyp.reject <- "reject H0"
} else {
  EXP.hyp.reject <- "do not reject H0"
}
print(EXP.hyp.reject)

## rejection rule: test stat must be more negative 
## RESULT: can reject H0 of non-stationarity for EXPEC2 


################################################################################
########################## CHECKING FOR COINTEGRATION ########################## 
################################################################################

# Run regression of one non-stationary on the other
reg <- lm(l.houseprice ~ INTEREST + l.unemployment + income.min.housingstock, data = data )
summary( reg )

# We want the residuals from this regression to see if cointegration exist   
data$u_hat_reg <- summary(reg)$residuals


###### Performing an EG-ADF test of the residual ###### 

# ADF test with automatic lag selection with up to 12 lags according to BIC
adf_u_hat <- ur.df( data$u_hat_reg, type="drift", lags = 12, selectlags = c("BIC") )
summary(adf_u_hat) 

hyp7.test_stat <- summary(adf_u_hat)@teststat[1]
print(hyp7.test_stat)

#### OBS!: test statistics cannot be compared with DF crit, must calcluate: 


# coefficients: no constant, no time trend
nc_coeffs <- data.frame(a  = c( 0.01,     0.05,    0.10),
                        b0 = c(-2.56574, -1.941,  -1.61682),
                        b1 = c(-2.2358,  -0.2686,  0.2656),
                        b2 = c(-3.627,   -3.365,  -2.714),
                        b3 = c(0,        31.223,  25.365)) 

# coefficients: constant, no time trend
c_coeffs <- data.frame( 
  N = c(1, 1, 1,
        2, 2, 2,
        3, 3, 3,
        4, 4, 4,
        5, 5, 5,
        6, 6, 6,
        7, 7, 7,
        8, 8, 8,
        9, 9, 9,
        10, 10, 10,
        11, 11, 11,
        12, 12, 12),
  a  = c(  0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10),
  b0 = c( -3.43035, -2.86154, -2.56677,
          -3.89644, -3.33613, -3.04445,
          -4.29374, -3.74066, -3.45218,
          -4.64332, -4.096,   -3.8102,
          -4.95756, -4.41519, -4.13157,
          -5.24568, -4.70693, -4.42501,
          -5.51233, -4.97684, -4.69648,
          -5.76202, -5.22924, -4.95007,
          -5.99742, -5.46697, -5.18897,
          -6.22103, -5.69244, -5.41533,
          -6.43377, -5.90714, -5.63086,
          -6.6379,  -6.11279, -5.83724 ),
  b1 = c( -6.5393,  -2.8903,  -1.5384,
          -10.9519, -6.1101,  -4.2412,
          -14.4354, -8.5631,  -6.2143,
          -18.1031, -11.2349, -8.3931,
          -21.8883, -14.0406, -10.7417,
          -25.6688, -16.9178, -13.1875,
          -29.576,  -19.9021, -15.7315,
          -33.5258, -23.0023, -18.3959,
          -37.6572, -26.2057, -21.1377,
          -41.7154, -29.4521, -24.0006,
          -46.0084, -32.8336, -26.9693,
          -50.2095, -36.2681, -29.9864 ),
  b2 = c( -16.786,  -4.234,   -2.809,
          -22.527,  -6.823,   -2.72,
          -33.195,  -10.852,  -3.718,
          -37.972,  -11.175,  -4.137,
          -45.142,  -12.575,  -3.784,
          -57.737,  -17.492,  -5.104,
          -69.398,  -22.045,  -6.922,
          -82.189,  -24.646,  -7.344,
          -87.365,  -26.627,  -9.484,
          -102.68,  -30.994,  -7.514,
          -106.809, -30.275,  -4.083,
          -124.156, -32.505,  -2.686 ),
  b3 = c( -79.433,  -40.040,  0,
          0,        0,        0,
          47.433,   27.982,   0,
          0,        0,        0,
          0,        0,        0,
          88.639,   60.007,   27.877,
          164.295,  110.761,  67.721,
          256.289,  144.479,  94.872,
          248.316,  176.382,  172.704,
          389.33,   251.016,  163.049,
          352.752,  249.994,  151.427,
          579.622,  314.802,  184.116 )
) 
# coefficients: constant and time trend
ct_coeffs <- data.frame(
  N = c(1, 1, 1,
        2, 2, 2,
        3, 3, 3,
        4, 4, 4,
        5, 5, 5,
        6, 6, 6,
        7, 7, 7,
        8, 8, 8,
        9, 9, 9,
        10, 10, 10,
        11, 11, 11,
        12, 12, 12 ),
  a  = c(  0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10,
           0.01,    0.05,     0.10 ),
  b0 = c(  -3.95877, -3.41049, -3.12705, 
           -4.32762, -3.78057, -3.49631, 
           -4.66305, -4.1189,  -3.83511, 
           -4.9694,  -4.42871, -4.14633, 
           -5.25276, -4.71537, -4.43422, 
           -5.51727, -4.98228, -4.70233, 
           -5.76537, -5.23299, -4.95405, 
           -6.00003, -5.46971, -5.19183, 
           -6.22288, -5.69447, -5.41738, 
           -6.43551, -5.90887, -5.63255, 
           -6.63894, -6.11404, -5.8385, 
           -6.83488, -6.31127, -6.0365 ),
  b1 = c(  -9.0531,  -4.3904,  -2.5856, 
           -15.4387, -9.5106,  -7.0815, 
           -18.7688, -11.8922, -9.0723, 
           -22.4694, -14.5876, -11.25, 
           -26.2183, -17.3569, -13.6078, 
           -29.976,  -20.305,  -16.1253, 
           -33.9165, -23.3328, -18.7352, 
           -37.8892, -26.4771, -21.4328, 
           -41.9496, -29.7152, -24.2882, 
           -46.1151, -33.0251, -27.2042, 
           -50.4287, -36.461,  -30.1995, 
           -54.7119, -39.9676, -33.2381 ),
  b2 = c(  -28.428,  -9.036,   -3.925, 
           -35.679,  -12.074,  -7.538, 
           -49.793,  -19.031,  -8.504, 
           -52.599,  -18.228,  -9.873, 
           -59.631,  -22.66,   -10.238, 
           -75.222,  -25.224,  -9.836, 
           -84.312,  -28.955,  -10.168, 
           -96.428,  -31.034,  -10.726, 
           -109.881, -33.784,  -8.584, 
           -120.814, -37.208,  -6.792, 
           -128.997, -36.246,  -5.163, 
           -139.8,   -37.021,  -6.606 ),
  b3 = c(  -134.155, -45.374,  -22.38, 
           0,        0,        21.892, 
           104.244,  77.332,  35.403, 
           51.314,   39.647,  54.109, 
           50.646,   91.359,  76.781, 
           202.253,  132.03,  94.272,
           245.394,  182.342, 120.575, 
           335.92,   220.165, 157.955, 
           466.068,  273.002, 169.891, 
           566.823,  346.189, 177.666, 
           642.781,  348.554, 210.338, 
           736.376,  406.051, 317.776 )
) 

# function to calculate critical value, inputs:
# - N: number of x-variables in the regression with stationary variables
# - what type: 
#   * nc (no constant, no time trend)
#   * c (constant, no time trend)
#   * ct (constant, time trend)
# - alpha: significance level
# - T: number of observations

crit_CT <- function(N, type, alpha, T){
  if(type == "nc"){ 
    # No constant, no trend
    b <- nc_coeffs[nc_coeffs$a == alpha,]
    return( b[["b0"]] + b[["b1"]]/T + b[["b2"]]/T^2 + b[["b3"]]/T^3 )
  } else if(type == "c"){  
    # Constant, but no trend 
    b <- c_coeffs[c_coeffs$N == N & c_coeffs$a == alpha,]
    return( b[["b0"]] + b[["b1"]]/T + b[["b2"]]/T^2 + b[["b3"]]/T^3 )
  } else if(type == "ct"){ 
    # Constant and time trend
    b <- ct_coeffs[c_coeffs$N == N & c_coeffs$a == alpha,]
    return( b[["b0"]] + b[["b1"]]/T + b[["b2"]]/T^2 + b[["b3"]]/T^3 )
  } else{
    stop("The type is unknown. Must be either 'nc', 'c' or 'ct'")
  }
}
T     <- nrow(data) #observations 
alpha <- 0.05 #significance level
hyp7.crit_value <- crit_CT(1, "c", alpha, T) # given a constant and no trend
print(hyp7.crit_value)

# calculating the test statistic
print(hyp7.test_stat)

# calculating the critical value at 5% significance level
print(hyp7.crit_value)

# Finding the rejection rule
if ( hyp7.test_stat < hyp7.crit_value ) {
  hyp7.reject <- "reject H0"
} else {
  hyp7.reject <- "do not reject H0"
}
print(hyp7.reject)

## Result: can reject H0 of non-stationarity
## A long run relationship thus exist, thus we can estimate an ECM...

################################################################################
############################### Estimate the ECM ############################### 
################################################################################

# ECM
ecm <- lm( d.houseprice ~ d.income + d.INTEREST + d.INTEREST_1 + EXPEC2 
           + l.houseprice_1 + INTEREST_1 + l.unemployment  + income.min.housingmass_1 
           + S1 + S2 + S3, data = data )
summary( ecm )


# Saving some results 
T <- nrow(ecm$model)

# Obtaining the residual
data$u <- summary(ecm)$residuals

################################################################################ 
#### Covariance matrix between coefficients to use in long-run relationship #### 
################################################################################ 

## Extracting the coefficients from the regression model
coefficients <- coef(ecm)

## Extract the residuals
residuals <- residuals(ecm)

## Calculate the variance of the residuals
residual_variance <- sum(residuals^2) / ecm$df.residual

## Design matrix 
design_matrix <- model.matrix(ecm)

# Calculate the covariance matrix
cov_matrix <- residual_variance * solve(t(design_matrix) %*% design_matrix)

show(cov_matrix) #extract those to houseprice_1

########### Get critical value to check if long term coefficients are significant

# Significance level (alpha)
alpha <- 0.05

# Degrees of freedom
df <- 119

# Calculate the critical value
critical_value <- qt(1 - alpha/2, df)

# Critical value, to compare against the long term coefficient
print(critical_value)

###############################################################################

# Extracting the predicted value:
ecm2 <- lm( l.houseprice ~ d.income + d.INTEREST + d.INTEREST_1 + EXPEC2 
            + l.houseprice_1 + INTEREST_1 + l.unemployment + income.min.housingmass_1  
            + S1 + S2 + S3, data = data )
summary( ecm2 )
predicted_value <- predict(ecm2)

##### Plot of predicted vs actual house prices
plot(data$month, data$l.houseprice, 
     type = "l", 
     col  = "blue",
     xlab = "Year",
     ylab = "",
     xaxt = "n",
    main = "")
lines(data$month, predicted_value, 
      type = "l", 
      col  = "red")
grid()
legend("right", legend = c("Actual", "Predicted"), 
       col = c("blue", "red"), lty = 1, cex = 0.8)
axis(side = 1, at = seq(min(data$month), max(data$month), by = "2 years"), 
     labels = format(seq(min(data$month), max(data$month), by = "2 years"), 
                     "%Y"))

################################################################################
############################## Diagnostic testing ############################## 
################################################################################

################# TESTING FOR HETEROSCEDASTICITY - WHITES Test #################

# obtaining the squared residual
data$u2 <- data$u^2

# y-hat
data$yhat <- predict(ecm)
data$yhat2 <- data$yhat^2

# auxilirary regression
unr_regr.hetr <- lm(u2 ~ yhat + yhat2, data=data)
summary(unr_regr.hetr)

# extracting the results 
T_hetr <- nrow(unr_regr.hetr$model) 
k <- ncol(unr_regr.hetr$model)
R2_hetr <- summary(unr_regr.hetr)$r.squared

# Chi-squared approach 
alpha <- 0.05
m <- 2

# test statistics
WT.test_stat <- R2_hetr * T_hetr
print (WT.test_stat)

# critical value
WT.crit_val <- qchisq(1-alpha, m)
print(WT.crit_val)

# rejection rule
if ( WT.test_stat > WT.crit_val ) {
  WT.hyp.reject <- "reject H0"
} else {
  WT.hyp.reject <- "do not reject H0"
}
print(WT.hyp.reject)

## Result: Cannot reject H0 of homoscedasticity: No heteroscedasticity.


############## TESTING FOR AUTOCORRELATION - Breusch Godfrey Test ##############

# Checking the plot
plot(data$u)

# Estimating the lagged estimated residuals
data$u1 <- c(0, data$u[1:T-1])
data$u2 <- c(0, data$u1[1:T-1])
data$u3 <- c(0, data$u2[1:T-1])
data$u4 <- c(0, data$u3[1:T-1])
data$u5 <- c(0, data$u4[1:T-1])
data$u6 <- c(0, data$u5[1:T-1])
data$u7 <- c(0, data$u6[1:T-1])
data$u8 <- c(0, data$u7[1:T-1])
data$u9 <- c(0, data$u8[1:T-1])
data$u10 <- c(0, data$u9[1:T-1])

ggplot(data=data, mapping=aes(x=data$month, y=data$u), color="blue") +
  geom_line(color="blue")+
  geom_point(color="blue")+
  labs(x = "", y = "")+
  theme_bw()

ggplot()+ 
  geom_point(data=data, mapping=aes(x=data$u1, y=data$u), color="blue")+
  labs(x = "resiudals(t-1)", y = "residuals(t)") +
  theme_bw()

 
# Running the auxilirary regression
aux_regr <- lm(u ~ d.income + d.INTEREST + d.INTEREST_1 + EXPEC + l.houseprice_1
               + INTEREST_1 + l.unemployment + income.min.housingmass_1  
               + S1 + S2 + S3
               + u1 + u2 + u3 + u4 + u5 + u6 + u7 + u8 + u9 + u10, data=data)
summary(aux_regr)

# Obtaining the R-squared 
R2_aux <- summary(aux_regr)$r.squared
print(R2_aux)

# Chi-squared version 
alpha <- 0.05
r <- 10

## H0: the lagged residual are jointly zero 
## H1: the lagged residuals are not jointly zero 

# Test statistics 
BG.test_stat <- R2_aux * (T-r)
print(BG.test_stat)

# Critical value 
BG.crit_val <- qchisq(1-alpha, r)
print(BG.crit_val)

# rejection rule
if ( BG.test_stat > BG.crit_val ) {
  BG.hyp.reject <- "reject H0"
} else {
  BG.hyp.reject <- "do not reject H0"
}
print(BG.hyp.reject)

############### Result: Can reject H0 -> There is autocorrelation. 


################## TESTING FOR NORMALITY - Jarques Bera test ##################
alpha <- 0.05

b1 <- mean(data$u^3) / mean(data$u^2)^(3/2)
print(b1)

b2 <- mean(data$u^4) / mean(data$u^2)^2
print(b2)

# Test statistics 
JB.test_stat <- T * (b1^2 / 6 + (b2-3)^2 / 24)
print(JB.test_stat)

# Critical value 
JB.crit_val <- qchisq(1-alpha, 2)
print(JB.crit_val)

# rejection rule
if ( JB.test_stat > JB.crit_val ) {
  JB.hyp.reject <- "reject H0"
} else {
  JB.hyp.reject <- "do not reject H0"
}
print(JB.hyp.reject)

############### Result: Cannot reject H0 -> The distribution is normal


################################################################################
#################################### Plots ##################################### 
################################################################################

##### The corrected explanatory variable 
plot(data$month, data$EXPEC2, #EXPEC if =>2004
     main="",
     type = "l", 
     col  = "blue",
     xlab = "", 
     ylab = "",
     ylim = c(-0.4,0.2),
     xaxt = "n")
grid()
axis(side = 1, at = seq(min(data$month), max(data$month), by = "2 years"), 
     labels = format(seq(min(data$month), max(data$month), 
                         by = "2 years"), "%Y"))
legend("bottomright", legend = c("Corrected expectation variable"), 
       col = c("blue"), lty = 1, cex = 0.8)


###### House price deflated with income ###### 

# Creating a new variable that is house price defalted with income
data$deflatedhouseprice <- data$houseprice / data$dispinc *100

### PLot deflated house prices
plot(data$month, data$deflatedhouseprice, 
     main="",
     type = "l", 
     col  = "blue",
     xlab = "", 
     ylab = "",
     xaxt = "n")
grid()
axis(side = 1, at = seq(min(data$month), max(data$month), by = "2 years"), 
     labels = format(seq(min(data$month), max(data$month), 
                         by = "2 years"), "%Y"))
legend("right", legend = c("House price deflated with total income"), 
       col = c("blue"), lty = 1, cex = 0.8)


###### Plotting the estimated model against acutal prices (in logs) ###### 

# 1. Calculate deviation
data$x <-  (data$l.houseprice - predicted_value) 
data$actual.min.predicted <- data$x

# 2. Plot deviation
plot(data$month, data$actual.min.predicted, 
     main="",
     type = "l", 
     col  = "blue",
     xlab = "", 
     ylab = "",
     xaxt = "n")
grid()
axis(side = 1, at = seq(min(data$month), max(data$month), by = "2 years"), 
     labels = format(seq(min(data$month), max(data$month), by = "2 years"), 
                     "%Y"))
legend("right", legend = c("Deviation"), 
       col = c("blue"), lty = 1, cex = 0.8)

